'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getStorageMetadata = exports.getStorageDownloadUrl = exports.STORAGE = exports.unsetUser = exports.setUser = exports.AUTH = exports.removeListener = exports.addListener = exports.LISTENERS = exports.removeQuery = exports.addQuery = exports.QUERIES = exports.COLLECTIONS = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _query = require('./modules/query');

var _actionTypes = require('./modules/actionTypes');

var COLLECTIONS = exports.COLLECTIONS = {
  MODIFY: (0, _actionTypes.createActionType)('collections/MODIFY'),
  MODIFY_ONE: (0, _actionTypes.createActionType)('collections/MODIFY_ONE'),
  REMOVE: (0, _actionTypes.createActionType)('collections/REMOVE')
};

var QUERIES = exports.QUERIES = _extends({}, (0, _actionTypes.createRequestActionTypes)('queries'), {
  ADD: (0, _actionTypes.createActionType)('queries/ADD'),
  REMOVE: (0, _actionTypes.createActionType)('queries/REMOVE')
});

var onIdle = function onIdle(fn) {
  return window.requestIdleCallback ? window.requestIdleCallback(fn) : setTimeout(fn, 1);
};

var _handleReceiveSnapshot = function _handleReceiveSnapshot(dispatch, query, queryId) {
  return function (snapshot) {
    var actions = [];
    if (snapshot.docChanges) {
      actions.push({ type: COLLECTIONS.MODIFY, payload: snapshot, meta: { query: query } });
    } else {
      actions.push({ type: COLLECTIONS.MODIFY_ONE, payload: snapshot, meta: { query: query } });
    }
    actions.push({ type: QUERIES.SUCCESS, payload: snapshot, meta: { queryId: queryId } });
    dispatch(actions);
  };
};

var addQuery = exports.addQuery = function addQuery(query) {
  var queryIdPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  return function (dispatch, getState) {
    var state = getState();
    var queries = state.queries;

    var queryId = '' + queryIdPrefix + (0, _query.getQueryId)(query);
    var collectionPath = '' + queryIdPrefix + (0, _query.getCollectionQueryPath)(query);

    var meta = { queryId: queryId };

    if (queryId !== collectionPath && collectionPath in queries) {
      var documentIds = (0, _query.getDocumentIdsForQuery)(query, state);
      return Promise.resolve(dispatch({ type: QUERIES.ADD, payload: documentIds, meta: meta }));
    }

    if (queryId in queries) {
      return Promise.resolve();
    }

    dispatch({ type: QUERIES.REQUEST, payload: query, meta: meta });

    return query.get().then(_handleReceiveSnapshot(dispatch, query, queryId)).catch(function (error) {
      dispatch({ error: true, type: QUERIES.FAILURE, payload: error, meta: meta });
    });
  };
};

var removeQuery = exports.removeQuery = function removeQuery(query) {
  var queryIdPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  return {
    type: QUERIES.REMOVE,
    payload: query,
    meta: { queryId: '' + queryIdPrefix + (0, _query.getQueryId)(query) }
  };
};

var LISTENERS = exports.LISTENERS = {
  ADD: (0, _actionTypes.createActionType)('listeners/ADD'),
  REMOVE: (0, _actionTypes.createActionType)('listeners/REMOVE')
};

var addListener = exports.addListener = function addListener(query) {
  var queryIdPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  return function (dispatch, getState) {
    var _getState = getState(),
        listeners = _getState.listeners;

    var queryId = '' + queryIdPrefix + (0, _query.getQueryId)(query);
    var collectionPath = '' + queryIdPrefix + (0, _query.getCollectionQueryPath)(query);

    if (queryId in listeners || collectionPath in listeners) {
      return Promise.resolve();
    }

    return new Promise(function (resolve) {
      onIdle(function () {
        var unsubscribe = query.onSnapshot(_handleReceiveSnapshot(dispatch, query, queryId));
        resolve(dispatch({ type: LISTENERS.ADD, payload: unsubscribe, meta: { queryId: queryId } }));
      });
    });
  };
};

var removeListener = exports.removeListener = function removeListener(query) {
  var queryIdPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  return function (dispatch, getState) {
    var queryId = '' + queryIdPrefix + (0, _query.getQueryId)(query);

    var _getState2 = getState(),
        listeners = _getState2.listeners;

    if (listeners[queryId]) {
      listeners[queryId].unsubscribe();
      return Promise.resolve(dispatch({
        type: LISTENERS.REMOVE,
        meta: { queryId: queryId }
      }));
    }

    return Promise.resolve();
  };
};

var _authQueryPrefix = 'auth|';
var AUTH = exports.AUTH = {
  CHANGE: (0, _actionTypes.createActionType)('auth/CHANGE')
};

var setUser = exports.setUser = function setUser(currentUser, userCollection) {
  return function (dispatch, getState, _ref) {
    var firestore = _ref.firestore;

    var actions = [];
    if (currentUser && userCollection) {
      var query = firestore.doc(userCollection + '/' + currentUser.uid);
      actions.push(addQuery(query, _authQueryPrefix));
      actions.push(addListener(query, _authQueryPrefix));
    }
    actions.push({ type: AUTH.CHANGE, payload: currentUser });
    return Promise.resolve(dispatch(actions));
  };
};

var unsetUser = exports.unsetUser = function unsetUser(uid, userCollection) {
  return function (dispatch, getState, _ref2) {
    var firestore = _ref2.firestore;

    var actions = [];
    if (uid) {
      if (userCollection) {
        var query = firestore.doc(userCollection + '/' + uid);
        actions.push(removeQuery(query, _authQueryPrefix));
        actions.push(removeListener(query, _authQueryPrefix));
        actions.push({ type: COLLECTIONS.REMOVE, payload: { id: uid }, meta: { query: query } });
      }
      actions.push({ type: AUTH.CHANGE });
      dispatch(actions);
    }
    return Promise.resolve();
  };
};

var STORAGE = exports.STORAGE = {
  URL: (0, _actionTypes.createRequestActionTypes)('storage/downloadUrl'),
  METADATA: (0, _actionTypes.createRequestActionTypes)('storage/metadata')
};

var getStorageDownloadUrl = exports.getStorageDownloadUrl = function getStorageDownloadUrl(reference) {
  return function (dispatch, getState) {
    var meta = { reference: reference };
    var state = getState();
    var ref = state.storage[reference.fullPath];
    if (ref && ref.downloadUrl) {
      return Promise.resolve(ref);
    }

    dispatch({ type: STORAGE.URL.REQUEST, meta: meta });
    return reference.getDownloadURL().then(function (url) {
      return dispatch({ type: STORAGE.URL.SUCCESS, payload: url, meta: meta });
    }).catch(function (error) {
      dispatch({ error: true, type: STORAGE.URL.FAILURE, payload: error, meta: meta });
      throw error;
    });
  };
};

var getStorageMetadata = exports.getStorageMetadata = function getStorageMetadata(reference) {
  return function (dispatch, getState) {
    var meta = { reference: reference };
    var state = getState();
    var ref = state.storage[reference.fullPath];
    if (ref && ref.metadata) {
      return Promise.resolve(ref);
    }

    dispatch({ type: STORAGE.METADATA.REQUEST, meta: meta });
    return reference.getMetadata().then(function (data) {
      return dispatch({ type: STORAGE.METADATA.SUCCESS, payload: data, meta: meta });
    }).catch(function (error) {
      dispatch({ error: true, type: STORAGE.METADATA.FAILURE, payload: error, meta: meta });
      throw error;
    });
  };
};
//# sourceMappingURL=actions.js.map